/*

  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. Neither the name of authors nor the names of its contributors
     may be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.
*/

#ifndef _context_h_included_
#define _context_h_included_

#include <iconv.h>

#include "uim.h"
#include "siod.h"

struct uim_im {
  char *lang;
  char *name;
  char *encoding;
};

struct cb_q {
  struct cb *first_cb;
  struct cb *tail_cb;
  int flushing;
};

struct uim_candidate_ {
  char *str;         /* candidate */
  char *heading_label;
  char *annotation;
  /* uim_pos part_of_speech; */
  /* int freq; */
  /* int freshness; */
  /* int formality; */
  /* char *src_dict; */
};

struct uim_context_ {
  void *ptr;
  /* internal id */
  int id;
  /* global id for helper */
  int global_id;

  struct uim_code_converter *conv_if;
  void *conv;
  char *encoding;
  /**/
  int commit_raw_flag;
  /**/
  int nr_modes;
  char **modes;
  /**/
  int mode;
  /**/
  char *proplabelstr;
  char *propstr;
  /**/
  int candidate_index;
  int nr_candidates;
  /**/
  void (*commit_cb)(void *ptr, char *str);
  /* preedit */
  void (*preedit_clear_cb)(void *ptr);
  void (*preedit_pushback_cb)(void *ptr, int attr, char *str);
  void (*preedit_update_cb)(void *ptr);
  /* mode list */
  void (*mode_list_update_cb)(void *ptr);
  /* mode */
  void (*mode_update_cb)(void *ptr, int);
  /* property list */
  void (*prop_list_update_cb)(void *ptr, char *str);
  /* property label */
  void (*prop_label_update_cb)(void *ptr, char *str);
  /* candidate window */
  void (*candidate_selector_activate_cb)(void *ptr, int nr, int index);
  void (*candidate_selector_select_cb)(void *ptr, int index);
  void (*candidate_selector_shift_page_cb)(void *ptr, int direciton);
  void (*candidate_selector_deactivate_cb)(void *ptr);
  /**/
  struct cb_q cb_q;
};

#define CAND_ACTIVATE_CB 1
#define CAND_SELECT_CB 2
#define CAND_DEACTIVATE_CB 3
#define COMMIT_CB 4
#define PREEDIT_CLEAR_CB 5
#define PREEDIT_PUSHBACK_CB 6
#define PREEDIT_UPDATE_CB 7
#define MODE_UPDATE_CB 8
#define MODE_LIST_UPDATE_CB 9
#define PROP_LABEL_UPDATE_CB 10
#define PROP_LIST_UPDATE_CB 11
#define CAND_SHIFT_PAGE_CB 12

struct cb {
  int type;
  char *str;
  int n1, n2;
  struct cb *next;
};

/**/
uim_context
uim_find_context(int id);
void uim_init_key_subrs();
void uim_init_util_subrs();
void uim_init_im_subrs();

char *uim_get_c_string(LISP str);
int uim_key_sym_to_int(LISP sym);

int uim_iconv_is_convertible(const char *tocode, const char *fromcode);
void *uim_iconv_create(const char *tocode, const char *fromcode);
char *uim_iconv_code_conv(iconv_t ic, const char *str);
void uim_iconv_release(void *obj);

void uim_eval_string(uim_context, char *str);
void uim_schedule_cb(uim_context, int type, char *str, int n1, int n2);

extern struct uim_im *uim_im_array;
extern int uim_nr_im;

#endif
